from PyObjCTools.TestSupport import TestCase
import InstallerPlugins


class TestInstallerPane(TestCase):
    def testConstants(self):
        self.assertEqual(InstallerPlugins.InstallerDirectionForward, 0)
        self.assertEqual(InstallerPlugins.InstallerDirectionBackward, 1)
        self.assertEqual(InstallerPlugins.InstallerDirectionUndefined, 2)

    def testMethods(self):
        self.assertResultIsBOOL(InstallerPlugins.InstallerPane.shouldExitPane_)
        self.assertArgIsBOOL(InstallerPlugins.InstallerPane.setNextEnabled_, 0)
        self.assertResultIsBOOL(InstallerPlugins.InstallerPane.nextEnabled)
        self.assertArgIsBOOL(InstallerPlugins.InstallerPane.setPreviousEnabled_, 0)
        self.assertResultIsBOOL(InstallerPlugins.InstallerPane.previousEnabled)
        self.assertResultIsBOOL(InstallerPlugins.InstallerPane.gotoNextPane)
        self.assertResultIsBOOL(InstallerPlugins.InstallerPane.gotoPreviousPane)



"""
Script for building the example.

Usage:
    python3 setup.py py2app
"""
from setuptools import setup

plist = {
    "InstallerSectionTitle": "PyRegistration",
    "NSMainNibFile": "Registration",
    "NSPrincipalClass": "InstallerSection",
    "CFBundleIdentifier": "com.MyCompany.Installer.example.Registration",
}

setup(
    name="Registration",
    plugin=["RegistrationPane.py"],
    data_files=["English.lproj"],
    options={"py2app": {"extension": ".bundle", "plist": plist}},
    setup_requires=[
        "py2app",
        "pyobjc-framework-Cocoa",
        "pyobjc-framework-InstallerPlugins",
    ],
)



"""
Some simple tests to check that the framework is properly wrapped.
"""
import objc
from PyObjCTools.TestSupport import TestCase
import InstallerPlugins


class TestInstallerPlugins(TestCase):
    def testClasses(self):
        self.assertHasAttr(InstallerPlugins, "InstallerSection")
        self.assertIsInstance(InstallerPlugins.InstallerSection, objc.objc_class)

        self.assertHasAttr(InstallerPlugins, "InstallerPane")
        self.assertIsInstance(InstallerPlugins.InstallerPane, objc.objc_class)



# This file is generated by objective.metadata
#
# Last update: Sat Jun 27 18:30:54 2020
#
# flake8: noqa

import objc, sys

if sys.maxsize > 2 ** 32:

    def sel32or64(a, b):
        return b


else:

    def sel32or64(a, b):
        return a


misc = {}
misc.update(
    {
        "InstallerSection_Private": objc.createStructType(
            "InstallerSection_Private", b"{InstallerSection_Private=}", []
        ),
        "InstallerState_Private": objc.createStructType(
            "InstallerState_Private", b"{InstallerState_Private=}", []
        ),
        "InstallerPane_Private": objc.createStructType(
            "InstallerPane_Private", b"{InstallerPane_Private=}", []
        ),
    }
)
constants = """$InstallerState_Choice_CustomLocation$InstallerState_Choice_Identifier$InstallerState_Choice_Installed$"""
enums = """$InstallerDirectionBackward@1$InstallerDirectionForward@0$InstallerDirectionUndefined@2$"""
misc.update({})
r = objc.registerMetaDataForSelector
objc._updatingMetadata(True)
try:
    r(b"InstallerPane", b"gotoNextPane", {"retval": {"type": "Z"}})
    r(b"InstallerPane", b"gotoPreviousPane", {"retval": {"type": "Z"}})
    r(b"InstallerPane", b"nextEnabled", {"retval": {"type": "Z"}})
    r(b"InstallerPane", b"previousEnabled", {"retval": {"type": "Z"}})
    r(b"InstallerPane", b"setNextEnabled:", {"arguments": {2: {"type": "Z"}}})
    r(b"InstallerPane", b"setPreviousEnabled:", {"arguments": {2: {"type": "Z"}}})
    r(b"InstallerPane", b"shouldExitPane:", {"retval": {"type": "Z"}})
    r(b"InstallerSection", b"gotoPane:", {"retval": {"type": "Z"}})
    r(b"InstallerSection", b"shouldLoad", {"retval": {"type": "Z"}})
    r(b"InstallerState", b"installStarted", {"retval": {"type": "Z"}})
    r(b"InstallerState", b"installSucceeded", {"retval": {"type": "Z"}})
    r(b"InstallerState", b"licenseAgreed", {"retval": {"type": "Z"}})
finally:
    objc._updatingMetadata(False)
expressions = {}

# END OF FILE



from PyObjCTools.TestSupport import TestCase
import InstallerPlugins


class TestInstallerState(TestCase):
    def testConstants(self):
        self.assertIsInstance(InstallerPlugins.InstallerState_Choice_Identifier, str)
        self.assertIsInstance(InstallerPlugins.InstallerState_Choice_Installed, str)
        self.assertIsInstance(
            InstallerPlugins.InstallerState_Choice_CustomLocation, str
        )

    def testMethods(self):
        self.assertResultIsBOOL(InstallerPlugins.InstallerState.licenseAgreed)
        self.assertResultIsBOOL(InstallerPlugins.InstallerState.installStarted)
        self.assertResultIsBOOL(InstallerPlugins.InstallerState.installSucceeded)



import Cocoa
import InstallerPlugins
import objc

# Important: be aware that informed end-users can remove your package's
# Plugins directory and/or change your package's InstallerSections.plist file
# to remove custom plugins from its flow.
# So plugins cannot serve as effective gatekeepers to prevent installation.


class RegistrationPane(InstallerPlugins.InstallerPane):
    uiFirstNameField = objc.IBOutlet()
    uiLastNameField = objc.IBOutlet()
    uiOrganizationField = objc.IBOutlet()
    uiSerialNumberField = objc.IBOutlet()

    def _entriesAreValid(self):
        """
        test all textfields to if they all have at least one character in them
        """
        if (
            len(self.uiFirstNameField.stringValue()) == 0
            or len(self.uiLastNameField.stringValue()) == 0
            or len(self.uiOrganizationField.stringValue()) == 0
            or len(self.uiSerialNumberField.stringValue()) == 0
        ):
            return False

        return True

    def _serialNumberIsValid(self):
        """
        perform a simple string compare to validate the serial number
        entered by the user
        """
        return self.uiSerialNumberField.stringValue() == "123-456-789"

    def _updateNextButtonState(self):
        """
        enable the 'Continue' button if '_entriesAreValid' returns 'True'
        """
        self.setNextEnabled_(self._entriesAreValid())

    def _localizedStringForKey_(self, key):
        """
        localization helper method:  This pulls localized strings from the
        plugin's bundle
        """
        return Cocoa.NSBundle.bundleForClass_(
            type(self)
        ).localizedStringForKey_value_table_(key, "", None)

    def title(self):
        """
        return the title of this pane
        """
        return self._localizedStringForKey_("Title")

    def didEnterPane_(self, direction):
        """
        pane's entry point: code called when user enters this pane
        """
        Cocoa.NSLog("DIDENTER")
        self._updateNextButtonState()

    def shouldExitPane_(self, direction):
        """
        called when user clicks "Continue" -- return value indicates
        if application should exit pane
        """
        if (
            direction == InstallerPlugins.InstallerDirectionForward
            and not self._serialNumberIsValid()
        ):
            self._updateNextButtonState()

            Cocoa.NSBeginInformationalAlertSheet(
                None,
                self._localizedStringForKey_("OK_BUTTON"),
                None,
                None,
                self.uiFirstNameField.window(),
                None,
                None,
                None,
                0,
                self._localizedStringForKey_("InvalidSerialNumberAlertMessage"),
            )
            return False

        return True

    def controlTextDidChange_(self, notification):
        """
        updates the state of the next button when the contents of the
        delegate textfields change
        """
        self._updateNextButtonState()



"""
Python mapping for the InstallerPlugins framework.

This module does not contain docstrings for the wrapped code, check Apple's
documentation for details on how to use these functions and classes.
"""
import sys

import AppKit
import objc
from InstallerPlugins import _metadata

sys.modules["InstallerPlugins"] = mod = objc.ObjCLazyModule(
    "InstallerPlugins",
    "com.apple.InstallerPlugins",
    objc.pathForFramework("/System/Library/Frameworks/InstallerPlugins.framework"),
    _metadata.__dict__,
    None,
    {
        "__doc__": __doc__,
        "__path__": __path__,
        "__loader__": globals().get("__loader__", None),
        "objc": objc,
    },
    (AppKit,),
)


del sys.modules["InstallerPlugins._metadata"]



from PyObjCTools.TestSupport import TestCase

import InstallerPlugins


class TestInstallerSection(TestCase):
    def testMethods(self):
        self.assertResultIsBOOL(InstallerPlugins.InstallerSection.shouldLoad)
        self.assertResultIsBOOL(InstallerPlugins.InstallerSection.gotoPane_)



"""
Wrappers for the "InstallerPlugins" framework on macOS. This framework
allows you to develop plugin's for the "Installer.app" application, and those
make it possible to add new functionality to ".pkg" and ".mpkg" installers
on macOS.

These wrappers don't include documentation, please check Apple's documention
for information on how to use this framework and PyObjC's documentation
for general tips and tricks regarding the translation between Python
and (Objective-)C frameworks
"""
from pyobjc_setup import setup

VERSION = "7.3"

setup(
    name="pyobjc-framework-InstallerPlugins",
    description="Wrappers for the framework InstallerPlugins on macOS",
    packages=["InstallerPlugins"],
    version=VERSION,
    install_requires=["pyobjc-core>=" + VERSION, "pyobjc-framework-Cocoa>=" + VERSION],
    long_description=__doc__,
)
